#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QSerialPort>
#include <QLabel>
#include <QString>
#include <QtSerialPort/QSerialPortInfo>
#include <QDebug>
#include <QTime>
//#include <QtMath>
//#include <qmath.h>


QSerialPort *serial;
QByteArray ReceivedData;
QString CalibFactor;
int portstatus = 0;
int ReceivedDataInt;
char mode_select;
//double F1=544753.0, F2= 296899.0, F3;
double F1, F2, F3;

//my function definations
void delay( int millisecondsToWait );


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    //Serial port
    serial = new QSerialPort(this);
    //Slot for async read fires class readData
    connect(serial,SIGNAL(readyRead()),this,SLOT(serialReceived()));
    setFixedSize(208, 435);
    ui->setupUi(this);
}

MainWindow::~MainWindow()
{
    delete ui;
}


void MainWindow::serialReceived(){ //Fires whenever data received
     double term1, term2, term3 , term4, Cx, Lx, Cref=2200.0;
     QString CX,LX;
     bool ok;
     //Read Calib factor and check for integral
     CalibFactor=ui->CalibLineEdit->text();
     int CalibFactorInt = CalibFactor.toInt(&ok);
         if (!ok){ // not an integer
             CalibFactorInt=100;
         }
         else {// an integer
            CalibFactorInt=CalibFactor.toInt();
         }
     Cref=Cref * CalibFactorInt/100;//Corrected Cref

     // Operate on Received data
     ReceivedData=(serial->readAll());

          if (ReceivedData.at(0) != 0x46) {
              mode_select = ReceivedData.at(0);
              ReceivedData = ReceivedData.remove(0, 1);
              ui->F3Label->setText("F3 = " +ReceivedData + " Hz");
              ReceivedDataInt = ReceivedData.toInt();

              // LC evaluation
              F3=ReceivedDataInt;
              if (F3>30000){
                  term1 = pow((F1 / F3), 2) - 1;
                  term2 = pow((F1 / F2), 2) - 1;

                  if (mode_select == 0x43) { //Capacitance Mode
                      Cx = (term1 / term2) * Cref; //double
                      if (Cx < 1000) {
                          CX= QString::number(Cx , 'f', 2);
                          ui->displayLabel->setText(CX + " pF");
                          }
                          else{
                          Cx=Cx/1000.0;
                          CX= QString::number(Cx, 'f', 2);
                          ui->displayLabel->setText(CX + " nF");
                          }

                  }
                  else if (mode_select == 0x4C) { //Inductance Mode
                      term3 = pow((1000000 / (2 * M_PI * F1)), 2);
                      term4 = 1/Cref;
                      Lx = term1 * term2 * term3 * term4*1000000.0;
                      if (Lx < 1000) {
                          LX = QString::number(Lx, 'f', 2);
                          ui->displayLabel->setText(LX + " uH");

                      }
                      else{
                          Lx=Lx/1000.0;
                          LX = QString::number(Lx, 'f', 2);
                          ui->displayLabel->setText(LX + " mH");
                      }
                  }
                }
              else{
                     ui->displayLabel->setText("Overrange");
                  }

            }
          else {
              ui->F3Label->setText(ReceivedData);
          }



}


void MainWindow::on_ConnectButton_clicked(){
    uint16_t vid = 0;
    uint16_t pid = 0;
    QString desc;
    //int port_check_status = 0;
    if (portstatus == 0){
       // port_check_status = 1;
        QString current_port;
        for (QSerialPortInfo port : QSerialPortInfo::availablePorts()){
            vid = port.vendorIdentifier();
            pid = port.productIdentifier();
            desc=port.description();
            if (vid == 1121){
                if(pid == 51){
                    if(desc=="Aj-LC"){
                        current_port.clear();
                        current_port.append(port.portName());
                        serial->setPortName(current_port);
                        //serial->setBaudRate(230400);
                        serial->setBaudRate(9600);
                        serial->setDataBits(QSerialPort::Data8);
                        serial->setParity(QSerialPort::NoParity);
                        serial->setStopBits(QSerialPort::OneStop);
                        serial->setFlowControl(QSerialPort::NoFlowControl);
                            if((serial->open(QIODevice::ReadWrite) == true)){
                         ui->ConnectButton->setText("Disconnect");
                            portstatus = 1;
                            //ui->StatusLabel->setText("Connected!");
                            //port_check_status = 2;
                            }
                        }
                    }
              }
        }
    }
    else{
        serial->close();
        ui->ConnectButton->setText("Connect");
        portstatus = 0;
        //ui->StatusLabel->setText("Disonnected!");
    }
}

void MainWindow::on_exitButton_clicked(){
    serial->close();
    QApplication::quit();
}

void MainWindow::on_ZeroPushButton_clicked(){
        ui->displayLabel->setText("Calibrating");
        serial->write("4"); //stop data
        delay(500);//wait for 1500 ms
        ReceivedData.clear();
        serial->write("0"); //Start Zeroing
        delay(4500);//wait for 4.5 seconds
        serial->write("1"); //Get F1
        delay(200);//wait for 200 ms

             /*if (ReceivedData.at(0) != 0x46) {
                  ReceivedData = ReceivedData.remove(0, 1);
             }
             else{
                 ReceivedData.clear();
                 serial->write("1"); //Get F1
                 delay(200);//wait for 200 ms
                 ReceivedData = ReceivedData.remove(0, 1);
             }*/

        ReceivedData = ReceivedData.remove(0, 1);
        ReceivedDataInt = ReceivedData.toInt();
        F1=ReceivedDataInt;
        ui->F1Label->setText("F1 = " + ReceivedData + " Hz");
        serial->write("2"); //Get F2
        delay(200);//wait for 200 ms
        ReceivedData = ReceivedData.remove(0, 1);
        ReceivedDataInt = ReceivedData.toInt();
        F2=ReceivedDataInt;
        ui->F2Label->setText("F2 = " +ReceivedData + " Hz");

        if((F1>500000)&&(F2<320000)&&(F2>270000)){
            ui->displayLabel->setText("Ready");
        }
        else{
            ui->displayLabel->setText("Repeat Zero");
          }

}

void MainWindow::on_RunPushButton_clicked(){
    ReceivedData.clear();
    serial->write("3");

}

void delay( int millisecondsToWait )
{
    QTime dieTime = QTime::currentTime().addMSecs( millisecondsToWait );
    while( QTime::currentTime() < dieTime )
    {
        QCoreApplication::processEvents( QEventLoop::AllEvents, 100 );
    }
}

